@@ -34,10 +34,13 @@ Lists three each good and bad things said about <thing>:
 
 =item get_karma($username)
 
-Returns either a string representing the total number of karma points for
-the passed C<$username> or the total number of karma points, an array of
-good karma comments, and an array of bad comma comments. The number of 
-good/bad comments returned can be configured with num_comments, below.
+Returns either a string representing the total number of karma
+points for the passed C<$username> or the total number of karma
+points and subroutine reference for good and bad karma comments.
+These references return the according karma levels when called in
+scalar context or a array of hash reference. Every hash reference
+has entries for the timestamp (timestamp), the giver (who) and the
+explanation string (reason) for its karma action.
 
 =item add_karma($object, $good, $reason, $who)
 
@@ -58,11 +61,19 @@ Defaults to 1; determines whether to respect selfkarmaing or not.
 
 =item num_comments
 
-Defaults to 3; number of good and bad comments to display on explanations.
+Defaults to 3; number of good and bad comments to display on
+explanations. Set this variable to 0 if you do not want to list
+reasons at all.
 
 =item show_givers
 
-Defaults to 1; whether to show who gave good or bad comments on explanations.
+Defaults to 1; whether to show who gave good or bad comments on
+explanations.
+
+=item randomize_reasons
+
+Defaults to 1; whether to randomize the order of reasons. If set
+to 0, the reasons are sorted in reversed chronological order.
 
 =back
 
@@ -82,9 +93,10 @@ use strict;
 
 sub init {
     my $self = shift;
-    $self->set("user_ignore_selfkarma", 1) unless defined($self->get("user_ignore_selfkarma"));
-    $self->set("user_num_comments", 3) unless defined($self->get("user_num_comments"));
-    $self->set("user_show_givers", 1) unless defined($self->get("user_show_givers"));
+    $self->set("user_ignore_selfkarma",  1) unless defined($self->get("user_ignore_selfkarma"));
+    $self->set("user_num_comments",      3) unless defined($self->get("user_num_comments"));
+    $self->set("user_show_givers",       1) unless defined($self->get("user_show_givers"));
+    $self->set("user_randomize_reasons", 1) unless defined($self->get("user_randomize_reasons"));
 }
 
 sub help {
@@ -126,33 +138,65 @@ sub told {
     my $nick = $self->bot->nick;
 
     my $tmp = $command;
-    $tmp =~ s!^$nick!!;
-    if ($tmp eq '++') {
-       return "Thanks!";
-    } elsif ($tmp eq '--') {
-       return "Pbbbbtt!";
+    if ($tmp =~ s!^$nick!! or $nick = $mess->{address}) {
+      if ($tmp eq '++') {
+         return "Thanks!";
+      } elsif ($tmp =~ /^--?$/) {
+         return "Pbbbbtt!";
+      }
     }
-    
-
-    if ($command eq "karma" and $param) {
-        return "$param has karma of ".$self->get_karma($param).".";
-
-    } elsif ($command eq "karma" and !$param) {
-        return $mess->{who} ." has karma of ".$self->get_karma($mess->{who}).".";
 
+    if ($command eq "karma") {
+	if ($param) {
+        	return "$param has karma of ".$self->get_karma($param).".";
+	} else {
+        	return $mess->{who} ." has karma of ".$self->get_karma($mess->{who}).".";
+	}
     } elsif ($command eq "explain" and $param) {
         $param =~ s/^karma\s+//i;
         my ($karma, $good, $bad) = $self->get_karma($param);
-        #$self->trim_list($good, $self->get("user_num_comments"));
-        #$self->trim_list($bad, $self->get("user_num_comments"));
-        my $reply  = "positive: ".scalar(@$good)."; ";
-           $reply .= "negative: ".scalar(@$bad)."; ";
+        my $reply  = "positive: ". $self->format_reasons($good)."; ";
+           $reply .= "negative: ". $self->format_reasons($bad) ."; ";
            $reply .= "overall: $karma.";
 
         return $reply;
     } 
 }
 
+sub format_reasons {
+  my ($self,$reason) = @_;
+  my $num_comments = $self->get('user_num_comments');
+
+  if ($num_comments == 0 ) {
+    return scalar($reason->());
+  }
+
+  my @reasons = $reason->();
+  my $num_reasons = @reasons;
+
+  if ($num_reasons == 0) {
+    return 'nothing';
+  } 
+
+  if ($num_reasons == 1) {
+    return ($self->maybe_add_giver(@reasons))[0];
+  } 
+
+  $self->trim_list(\@reasons,$num_comments);
+  return join(', ', $self->maybe_add_giver(@reasons));
+}
+
+sub maybe_add_giver {
+    my ($self,@reasons) = @_;
+    if ($self->get('user_show_givers')) {
+      # adding a (user) string to the all reasons
+      return map { $_->{reason} . ' (' . $_->{who} . ')' } @reasons
+    } else {
+      # just returning the reason string of the reason hash referenes
+      return map { $_->{reason} } @reasons
+    }
+}
+
 sub get_karma {
     my ($self, $object) = @_;
     $object = lc($object);
@@ -161,15 +205,28 @@ sub get_karma {
     my @changes = @{ $self->get("karma_$object") || [] };
 
     my (@good, @bad);
-    my $karma = 0;
+    my $karma    = 0;
+    my $positive = 0;
+    my $negative = 0;
 
     for my $row (@changes) {
-        my $who = $self->get("user_show_givers") ? " (".$row->{who}.")" : undef; 
-        if ($row->{positive}) { $karma++; push(@good, $row->{reason}.$who) }
-        else                  { $karma--; push(@bad, $row->{reason}.$who)  }
-    }
+        # just push non empty reasons on the array
+        my $reason = $row->{reason};
+        if ($row->{positive}) { $positive++; push(@good, $row) if $reason}
+        else                  { $negative++; push(@bad,  $row) if $reason}
+      }
+    $karma = $positive - $negative;
 
-    return wantarray() ? ($karma, \@good, \@bad) : $karma;
+    # The subroutine references return differant values when called.
+    # If they are called in scalar context, they return the overall
+    # positive or negative karma, but when called in list context you
+    # get an array of hash references with all non empty reasons back.
+
+    return wantarray() 
+           ? ( $karma, sub { return wantarray ? @good : $positive },
+	               sub { return wantarray ? @bad  : $negative })
+	   :   $karma
+	   ;
 }
 
 sub add_karma {
@@ -183,7 +240,17 @@ sub add_karma {
 
 sub trim_list {
     my ($self, $list, $count) = @_;
-    fisher_yates_shuffle($list);
+
+    # If radomization isn't requested we just return the reasons
+    # in reversed cronological order
+
+    if ($self->get('user_randomize_reasons')){
+      fisher_yates_shuffle($list);
+    } 
+    else {
+      @$list = reverse sort { $b->{timestamp} cmp $a->{timestamp} } @$list;
+    }
+
     if (scalar(@$list) > $count) {
         @$list = splice(@$list, 0, $count);
     }
